import sys
from pathlib import Path

# Dynamically add the package directory to sys.path
package_path = str(Path(__file__).parent.absolute())
if package_path not in sys.path:
    sys.path.insert(0, package_path)



"""C.AI specific constants for the Hermes engine."""

NARRATOR_NAME = "narrator"
DEFAULT_USERNAME = "user"
TRUNCATION_STEPS_CHOICES = [250, 500, 750, 1000, 1250, 1500]
PRETRUNCATION_TOKENS_PER_MESSAGE = 10
BOD = "<|beginningofdialog|>"
BOM = "<|beginningofmessage|>"
EOM = "<|endofmessage|>"



"""Decorators for tokenizer."""

import time

from metrics import Metrics


def monitor(func):
    """Decorator for monitoring the latency of a function."""

    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        stop = time.time()
        Metrics().FUNCTION_LATENCY.labels(func.__qualname__).observe(stop - start)
        return result

    return wrapper



import unittest

from hermes.hermes_cai.contrib.lm_prefix_utils import parse_character_definition, \
    NARRATOR_NAME

CHARACTER_NAME = "Character-Name"
USER_NAME = "User-Name"
AUTHORS = {
    "char": CHARACTER_NAME,
    "user": USER_NAME,
}


class TestParseCharacterDefinition(unittest.TestCase):
    def test_inject_placeholders(self):
        definition = """
{{char}} 1
{{user}} 2
{{random_user_1}} 3
{{random_user_2}} 4
{{some name}} 5
{{char}} 6
{{user}} 7
{{random_user_1}} 8
{{random_user_2}} 9
{{some name}} 6
        """
        v0, _ = parse_character_definition(definition, AUTHORS, version=0)
        v1, _ = parse_character_definition(definition, AUTHORS, version=1)
        self.assertEqual(v0, v1)

    def test_parse(self):
        test_cases = [
            {
                "name": "Empty description",
                "definition": "",
                "result": []
            },
            {
                "name": "Simple description",
                "definition": "Some description here",
                "result": [
                    {"src": NARRATOR_NAME, "text":  "Some description here"},
                ]
            },
            {
                "name": "Description with placeholders",
                "definition": "Some description here, {{char}} and {{user}}",
                "result": [
                    {"src": NARRATOR_NAME, "text": f"Some description here, {CHARACTER_NAME} and {USER_NAME}"},
                ]
            },
            {
                "name": "Simple dialog",
                "definition": "{{char}}: msg1\n{{user}}: msg2\n{{char}}: msg3\nEND_OF_DIALOG",
                "result": [
                    {"src": CHARACTER_NAME, "text": "msg1"},
                    {"src": USER_NAME, "text": "msg2"},
                    {"src": CHARACTER_NAME, "text": "msg3"},
                ]
            },
            {
                "name": "Multiple dialogs",
                "definition": "{{char}}: msg1\n{{user}}: msg2\n{{char}}: msg3\nEND_OF_DIALOG\n{{char}}: msg4\n{{user}}: msg5\n{{char}}: msg6\nEND_OF_DIALOG",
                "result": [
                    {"src": CHARACTER_NAME, "text": "msg1"},
                    {"src": USER_NAME, "text": "msg2"},
                    {"src": CHARACTER_NAME, "text": "msg3"},
                    {"src": CHARACTER_NAME, "text": "msg4"},
                    {"src": USER_NAME, "text": "msg5"},
                    {"src": CHARACTER_NAME, "text": "msg6"},
                ]
            },
            {
                "name": "Dialog without END_OF_DIALOG",
                "definition": "{{char}}: msg1\n{{user}}: msg2\n{{char}}: msg3",
                "result": [
                    {"src": NARRATOR_NAME, "text": f"{CHARACTER_NAME}: msg1\n{USER_NAME}: msg2\n{CHARACTER_NAME}: msg3"},
                ]
            },
            {
                "name": "Dialog and description",
                "definition": "{{char}}: msg1\n{{user}}: msg2\n{{char}}: msg3\nEND_OF_DIALOG\nSome description here",
                "result": [
                    {"src": CHARACTER_NAME, "text": "msg1"},
                    {"src": USER_NAME, "text": "msg2"},
                    {"src": CHARACTER_NAME, "text": "msg3"},
                    {"src": NARRATOR_NAME, "text": "Some description here"},
                ]
            },
            {
                "name": "Description and dialog",
                "definition": "Some description here\n{{char}}: msg1\n{{user}}: msg2\n{{char}}: msg3\nEND_OF_DIALOG",
                "result": [
                    {"src": NARRATOR_NAME, "text": "Some description here"},
                    {"src": CHARACTER_NAME, "text": "msg1"},
                    {"src": USER_NAME, "text": "msg2"},
                    {"src": CHARACTER_NAME, "text": "msg3"},
                ]
            },
            {
                "name": "Multilines",
                "definition": "\n\nSome description here\nEven more description\n\n{{char}}: msg1-1\nmsg1-2\n{{user}}: msg2-1\nmsg2-2\n\n{{char}}: msg3-1\nmsg3-2\nEND_OF_DIALOG\n\n",
                "result": [
                    {"src": NARRATOR_NAME, "text": "Some description here\nEven more description"},
                    {"src": CHARACTER_NAME, "text": "msg1-1\nmsg1-2"},
                    {"src": USER_NAME, "text": "msg2-1\nmsg2-2"},
                    {"src": CHARACTER_NAME, "text": "msg3-1\nmsg3-2"},
                ]
            },
            {
                "name": "Multilines for windows",
                "definition": "\r\n\r\nSome description here\r\nEven more description\r\n\r\n{{char}}: msg1-1\r\nmsg1-2\r\n{{user}}: msg2-1\r\nmsg2-2\r\n\r\n{{char}}: msg3-1\r\nmsg3-2\r\nEND_OF_DIALOG\r\n\r\n",
                "result": [
                    {"src": NARRATOR_NAME, "text": "Some description here\nEven more description"},
                    {"src": CHARACTER_NAME, "text": "msg1-1\nmsg1-2"},
                    {"src": USER_NAME, "text": "msg2-1\nmsg2-2"},
                    {"src": CHARACTER_NAME, "text": "msg3-1\nmsg3-2"},
                ]
            },
            {
                "name": "Complex",
                "definition": """
desc1
desc2
{{char}}: msg1-1
msg1-2

{{user}}:
msg2-1
msg2-2
{{char}}:
msg3-1

msg3-2
END_OF_DIALOG

desc4

desc5
{{char}}: msg4-1
msg4-2

{{user}}:
msg5-1
msg5-2
{{char}}:
msg6-1

msg6-2
END_OF_DIALOG
desc6
""",
                "result": [
                    {"src": NARRATOR_NAME, "text": "desc1\ndesc2"},
                    {"src": CHARACTER_NAME, "text": "msg1-1\nmsg1-2"},
                    {"src": USER_NAME, "text": "msg2-1\nmsg2-2"},
                    {"src": CHARACTER_NAME, "text": "msg3-1\nmsg3-2"},
                    {"src": NARRATOR_NAME, "text": "desc4\ndesc5"},
                    {"src": CHARACTER_NAME, "text": "msg4-1\nmsg4-2"},
                    {"src": USER_NAME, "text": "msg5-1\nmsg5-2"},
                    {"src": CHARACTER_NAME, "text": "msg6-1\nmsg6-2"},
                    {"src": NARRATOR_NAME, "text": "desc6"},
                ]
            },
        ]
        for case in test_cases:
            with self.subTest(case["name"]) as subtest:
                definition = case["definition"]
                expected = case["result"]
                v1, _ = parse_character_definition(definition, AUTHORS, version=1)
                self.assertEqual(v1, expected)


if __name__ == "__main__":
    unittest.main()



from google.cloud import storage


class GCSClient:
    _gcs_client = None

    @classmethod
    def get_client(cls):
        if cls._gcs_client is None:
            cls._gcs_client = storage.Client()
        return cls._gcs_client



"""Helper functions for Hermes CAI template rendering."""

import re
from dataclasses import dataclass

from constants import DEFAULT_USERNAME, NARRATOR_NAME, PRETRUNCATION_TOKENS_PER_MESSAGE

# pylint: disable=unused-import
from contrib.lm_prefix_utils import get_character_priming,get_scene_definition  # noqa: F401
from exceptions import MissingContextDataError


@dataclass(slots=True, frozen=False)
class ChatContextMessage:
    """Chat Context Message."""

    author: str
    text: str
    # Unspecified: 0
    # HumanMessage: 1
    # CharacterMessage: 2
    # SystemMessage: 3
    type: int
    is_pinned: bool
    is_summary: bool
    attachments_content: str
    # Optional.
    is_safety_truncated: bool | None = None
    attachments_content_list: list[str] | None = None
    chat_id: str | None = None
    turn_id: str | None = None
    candidate_id: str | None = None


def maybe_inject_narrator(message: str, default_author: str = NARRATOR_NAME) -> str:
    """Inject narrator into the message if applicable."""
    if re.match(r"^[\w-]+:", message):
        return message
    return f"{default_author}: {message}"


def raise_missing_context_data(key: str):
    """Raise missing data from jinja context."""
    raise MissingContextDataError(f"Missing required key in jinja context: {key=}")


def canonicalize_user_name(name: str | None) -> str:
    """Makes name format consistent with author names we use in training data."""
    # The "-" is used in upstream components and should be overriden to default value.
    if name is None or not name or name == "-":
        return DEFAULT_USERNAME
    return "-".join(name.split())


def pretruncate_messages(
    messages: list[ChatContextMessage], token_limit: int
) -> list[ChatContextMessage]:
    """Pretruncates messages for Hermes generation."""
    message_truncation_step = token_limit // PRETRUNCATION_TOKENS_PER_MESSAGE
    while len(messages) > 2 * message_truncation_step:
        messages = messages[message_truncation_step:]
    return messages



"""Setup file for the Hermes package."""

from pathlib import Path

from setuptools import find_packages, setup

# Read the version from the VERSION file
with (Path(__file__).parent.absolute() / "VERSION").open() as version_file:
    version = version_file.read().strip()

long_description: str
with Path("README.md").open() as readme_file:
    long_description = readme_file.read()

setup(
    name="hermes-cai",
    version=version,
    packages=find_packages(include=["hermes_cai", "hermes_cai.*"]),
    include_package_data=True,
    package_data={"hermes_cai": ["templates/*", "contrib/vocab/*"]},
    install_requires=[
        "prompt-poet==0.0.49",
        "prometheus-client==0.20.0",
        "pydantic>=2.7.4",
    ],
    author="James Groeneveld",
    author_email="james@character.ai",
    description="The simplest way of using control flow (like if statements and for loops) to build production-grade prompts for LLMs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/character-tech/chat-stack",
    python_requires=">=3.10",
    license="MIT",
)
